import re

from nltk import WordNetLemmatizer
from nltk.corpus import stopwords

from sekg.util.code import CodeElementNameUtil
from .base import Preprocessor


class CodeDocPreprocessor(Preprocessor):
    CLEAN_PATTERN = re.compile(r'[^a-zA-Z_]')
    STOPWORDS = set(stopwords.words('english'))
    code_element_util = CodeElementNameUtil()
    lemmatizer = WordNetLemmatizer()
    lemma_map = {}

    def extract_words_for_query(self, query):
        return set(self.clean(query))

    def lemma(self, word):
        if word in self.lemma_map:
            return self.lemma_map[word]

        lemma = self.lemmatizer.lemmatize(self.lemmatizer.lemmatize(word, "n"), "v")

        if lemma != word:
            self.lemma_map[word] = lemma
        return lemma

    def clean(self, text):
        """
        return a list of token from the text, only remove stopword and lemma the word
        :param text: the text need to preprocess
        :return: list of str
        """

        clean_text = re.sub(self.CLEAN_PATTERN, " ", text)
        old_words_set = set([word.lower() for word in clean_text.split(" ") if word and word not in self.STOPWORDS])
        clean_text = self.code_element_util.split_camel_name_and_underline(clean_text)
        if not clean_text:
            return []
        new_words = clean_text.lower().split(" ")
        new_words = [self.lemma(word) for word in new_words
                     if word and word not in self.STOPWORDS]

        new_words_set = set(new_words)

        for old_word in old_words_set:
            if old_word not in new_words_set and old_word not in self.lemma_map:
                new_words.append(old_word)

        return new_words



class CodeConstant:
    LABEL_CODE_ELEMENT = "code_element"


class DomainConstant:
    LABEL_DOMAIN_TERM = "domain term"
    PRIMARY_PROPERTY_NAME = "term_name"


class OperationConstance:
    LABEL_OPERATION = "operation"
    PRIMARY_PROPERTY_NAME = "operation_name"


class PropertyConstant:
    COMMENT = "comment"
    INSIDE_COMMENT = "inside_comment"
    STRING_LITERAL_EXPR = "string_literal_expr"
    DESCRIPTION = "short_description"
    ALIAS = "alias"
    LEMMA = "lemma"



from unittest import TestCase

from sekg.ir.preprocessor.spacy import SpacyTextPreprocessor


class TestSpacyTextPreprocessor(TestCase):
    def test_extract_words_for_query(self):
        preprocessor = SpacyTextPreprocessor()
        test_case_list = [
            (
                "This is a ArrayList, it contains all of the classes for creating user interfaces and for painting graphics and images."
                , ['ArrayList',
                   'class',
                   'user',
                   'interface',
                   'painting',
                   'graphic',
                   'image',
                   'contain',
                   'create']
            ),

            (
                "how to get File's MD5 checksum",
                ['File', 'md5', 'checksum']

            )

        ]

        for old_str, new_str in test_case_list:
            team = preprocessor.extract_words_for_query(old_str)
            self.assertEqual(team, new_str)

    def test_clean(self):

        test_case_list = [
            (
                "This is a ArrayList, it contains all of the classes for creating user interfaces and for painting graphics and images."
                ,
                ['arraylist',
                 'contain',
                 'class',
                 'create',
                 'user',
                 'interface',
                 'painting',
                 'graphic',
                 'image']
            ),

            ("how to get File's MD5 checksum",
             ["file", "md5", "checksum"])
        ]
        preprocessor = SpacyTextPreprocessor()

        for old_str, keywords in test_case_list:
            self.assertEqual(keywords, preprocessor.clean(old_str))
